package ijeoma.fabrica.benetton.window;

import processing.core.PApplet;
import processing.core.PFont;
import processing.core.PImage;

public class BWindow {
	PFont f;
	PApplet parent;
	private int id;
	private float x, y, width, height;
	private int rows, columns;
	private float viewableX, viewableY;
	private float viewableWidth, viewableHeight;
	private float scale;
	private boolean hasCamera;

	public static String SCALE_NORMAL = "scaleNormal";
	public static String SCALE_WIDTH = "scaleWidth";
	public static String SCALE_HEIGHT = "scaleHeight";

	String scaleMode = SCALE_NORMAL;

	public static String V_ALIGN_TOP = "alignTop";
	public static String V_ALIGN_CENTER = "alignCenter";
	public static String V_ALIGN_BOTTOM = "alignBottom";

	String verticalAlignMode = V_ALIGN_CENTER;

	public static String H_ALIGN_LEFT = "alignLeft";
	public static String H_ALIGN_CENTER = "alignCenter";
	public static String H_ALIGN_RIGHT = "alignRight";

	String horizontalAlignMode = H_ALIGN_CENTER;

	float sketchWidth, sketchHeight;
	float sketchScale;
	float sketchToWindowXOffset, sketchToWindowYOffset;

	float sketchToWindowRatioX, sketchToWindowRatioY;
	float sketchToWindowRatioWidth, sketchToWindowRatioHeight;

	float sketchToWindowScale;

	float sketchToWindowX, sketchToWindowY;
	float sketchToWindowWidth, sketchToWindowHeight;
	float sketchScaledXOffset, sketchScaledYOffset;

	BWindow(PApplet _parent, int _id, float _x, float _y, float _width,
			float _height, int _rows, int _columns, int _hasCamera) {
		parent = _parent;

		f = parent.createFont("Arial", 32);

		id = _id;

		x = _x;
		y = _y;

		width = _width;
		height = _height;

		rows = _rows;
		columns = _columns;

		float rowsMax = (((rows / 5f) > (int) (rows / 5f)) ? (int) (rows / 5f) + 1
				: (int) (rows / 5f)) * 5;
		float columnsMax = (((columns / 5f) > (int) (columns / 5f)) ? (int) (columns / 5f) + 1
				: (int) (columns / 5f)) * 5;

		viewableX = x;// + (width * (columnsMax - columns) / columnsMax));
		viewableY = y + (height * ((rowsMax - rows) / rowsMax));

		viewableWidth = width * (columns / columnsMax);
		viewableHeight = height * (rows / rowsMax);

		scale = 1;

		hasCamera = (_hasCamera == 1) ? true : false;
	}

	void drawBounds() {
		parent.strokeCap(PApplet.ROUND);
		parent.strokeWeight(4f);

		parent.stroke(255, 0, 0);
		// parent.fill(0);
		parent.noFill();
		parent.rect(getScaledX(), getScaledY(), getScaledWidth(),
				getScaledHeight());

		parent.strokeWeight(1);
		parent.noStroke();
		parent.noFill();
	}

	void drawViewableBounds() {
		parent.strokeCap(PApplet.ROUND);
		parent.strokeWeight(2f);

		parent.stroke(0, 255, 0);
		// parent.fill(255);
		parent.noFill();
		parent.rect(getScaledViewableX(), getScaledViewableY(),
				getScaledViewableWidth(), getScaledViewableHeight());

		parent.strokeWeight(1);
		parent.noStroke();
		parent.noFill();
	}

	void drawId() {
		parent.noStroke();
		parent.fill(0, 255, 0);
		parent.textFont(f, 32f);
		parent.text(id, getScaledViewableX() + getScaledViewableWidth() / 2
				- parent.textWidth(id + "") / 2, getScaledViewableY()
				+ getScaledViewableHeight() / 2);

		parent.strokeWeight(1);
		parent.noStroke();
		parent.noFill();
	}

	public boolean hasCamera() {
		return hasCamera;
	}

	private float[] getScaledSketchPosition(float _x, float _y) {
		sketchToWindowRatioX = getViewableWidth() / _y;
		sketchToWindowRatioY = getViewableWidth() / _x;
		sketchToWindowRatioWidth = getViewableWidth() / sketchWidth;
		sketchToWindowRatioHeight = getViewableHeight() / sketchHeight;

		sketchToWindowScale = sketchToWindowRatioWidth < sketchToWindowRatioHeight ? sketchToWindowRatioWidth
				: sketchToWindowRatioHeight;

		sketchToWindowX = _x * sketchToWindowScale;
		sketchToWindowY = _y * sketchToWindowScale;
		sketchToWindowWidth = sketchWidth * sketchToWindowScale;
		sketchToWindowHeight = sketchHeight * sketchToWindowScale;

		if (horizontalAlignMode == H_ALIGN_LEFT)
			sketchToWindowXOffset = 0;
		else if (horizontalAlignMode == H_ALIGN_CENTER)
			sketchToWindowXOffset = getViewableWidth() / 2f
					- (sketchToWindowWidth * sketchScale) / 2f;
		else if (horizontalAlignMode == H_ALIGN_RIGHT)
			sketchToWindowXOffset = getViewableWidth() - sketchToWindowWidth
					* sketchScale;

		if (verticalAlignMode == V_ALIGN_TOP)
			sketchToWindowYOffset = 0;
		else if (verticalAlignMode == V_ALIGN_CENTER)
			sketchToWindowYOffset = getViewableHeight() / 2f
					- (sketchToWindowHeight * sketchScale) / 2f;
		else if (verticalAlignMode == V_ALIGN_BOTTOM)
			sketchToWindowYOffset = getViewableHeight() - (sketchToWindowHeight
					* sketchScale);

		sketchScaledXOffset = sketchToWindowXOffset;
		sketchScaledYOffset = sketchToWindowYOffset;

		return new float[] {
				(getViewableX() + sketchToWindowX + sketchScaledXOffset)
						* scale,
				(getViewableY() + sketchToWindowY + sketchScaledYOffset)
						* scale };
	}

	private float getScaledSketchScale(float _scale) {
		sketchScale = _scale;

		sketchToWindowRatioWidth = getViewableWidth() / sketchWidth;
		sketchToWindowRatioHeight = getViewableHeight() / sketchHeight;

		if (scaleMode == SCALE_NORMAL)
			sketchToWindowScale = sketchToWindowRatioWidth < sketchToWindowRatioHeight ? sketchToWindowRatioWidth
					: sketchToWindowRatioHeight;
		if (scaleMode == SCALE_WIDTH)
			sketchToWindowScale = sketchToWindowRatioWidth;
		if (scaleMode == SCALE_HEIGHT)
			sketchToWindowScale = sketchToWindowRatioHeight;

		return sketchToWindowScale * scale * _scale;
	}

	public void setupTransform(float _width, float _height) {
		sketchWidth = _width;
		sketchHeight = _height;

		sketchToWindowRatioX = sketchToWindowRatioY = 0;
		sketchToWindowRatioWidth = sketchToWindowRatioHeight = 0;

		sketchToWindowScale = 0;

		sketchToWindowX = sketchToWindowY = 0;
		sketchToWindowWidth = sketchToWindowHeight = 0;

		sketchToWindowXOffset = sketchToWindowYOffset = 0;

		transform(0, 0, 1);
	}

	public void translate(float _x, float _y) {
		// map world-sized object, to the local-sized window display
		float[] scaledMappedPosition = getScaledSketchPosition(_x, _y);

		parent.rectMode(PApplet.CORNER);
		parent.imageMode(PApplet.CORNER);
		parent.translate(scaledMappedPosition[0], scaledMappedPosition[1]);
	}

	public void scale(float _scale) {
		// map world-sized object, to the local-sized window display
		float scaledMappedScale = getScaledSketchScale(_scale);

		parent.scale(scaledMappedScale);
	}

	public void transform(float _x, float _y, float _scale) {
		translate(_x, _y);
		scale(_scale);
	}

	public int getId() {
		return id;
	}

	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	public float getWidth() {
		return width;
	}

	public float getHeight() {
		return height;
	}

	public float getViewableX() {
		return viewableX;
	}

	public float getViewableY() {
		return viewableY;
	}

	public float getViewableWidth() {
		return viewableWidth;
	}

	public float getViewableHeight() {
		return viewableHeight;
	}

	void setScale(float _value1) {
		scale = _value1;
	}

	public float getScale() {
		return scale;
	}

	public float getScaledX() {
		return x * scale;
	}

	public float getScaledY() {
		return y * scale;
	}

	public float getScaledWidth() {
		return width * scale;
	}

	public float getScaledHeight() {
		return height * scale;
	}

	public float getScaledViewableX() {
		return viewableX * scale;
	}

	public float getScaledViewableY() {
		return viewableY * scale;
	}

	public float getScaledViewableWidth() {
		return viewableWidth * scale;
	}

	public float getScaledViewableHeight() {
		return viewableHeight * scale;
	}

	public void setScaleMode(String _scaleMode) {
		scaleMode = _scaleMode;
	}

	public void setHorizantalAlignMode(String _alignMode) {
		horizontalAlignMode = _alignMode;
	}

	public void setVerticalAlignMode(String _alignMode) {
		verticalAlignMode = _alignMode;
	}
}
