package ijeoma.fabrica.benetton.window;

import processing.core.PApplet;
import processing.core.PImage;
import processing.video.Movie;
import processing.xml.XMLElement;

public class BWindowManager {
	private PApplet parent;

	// BWindowImage logo;
	// BWindowMovie intro, outro

	private String name = "";

	private float x = 0, y = 0;
	private float width = 0, height = 0;

	private int rows = 0, columns = 0;

	private float scale = 1;

	PImage logo;
	String logoSrc = "";
	float logoX, logoY, logoWidth, logoHeight = 0;

	PImage splash;
	String splashSrc = "";
	float splashX, splashY, splashWidth, splashHeight = 0;

	Movie intro;
	String introSrc = "";
	float introX, introY, introWidth, introHeight = 0;

	Movie outro;
	String outroSrc = "";
	float outroX, outroY, outroWidth, outroHeight = 0;

	private BWindow[] windows;

	private int cameraWindowId = 0;

	public BWindowManager(PApplet _parent, float _x, float _y) {
		parent = _parent;

		x = _x;
		y = _y;
	}

	public void load(String _fileName) {
		// PApplet.println("begin BWindowManger.load(" + _fileName + ")");

		XMLElement xml = new XMLElement(parent, _fileName);

		name = xml.getAttribute("name");

		XMLElement logoNode = xml.getChild("logo");

		logoSrc = logoNode.getStringAttribute("src");
		logoX = logoNode.getFloatAttribute("logoX");
		logoY = logoNode.getFloatAttribute("logoY");
		logoWidth = logoNode.getFloatAttribute("logoWidth");
		logoHeight = logoNode.getFloatAttribute("logoHeight");

		XMLElement splashNode = xml.getChild("splash");

		splashSrc = splashNode.getStringAttribute("src");
		splashX = splashNode.getFloatAttribute("splashX");
		splashY = splashNode.getFloatAttribute("splashY");
		splashWidth = splashNode.getFloatAttribute("splashWidth");
		splashHeight = splashNode.getFloatAttribute("splashHeight");

		XMLElement introNode = xml.getChild("intro");

		introSrc = introNode.getStringAttribute("src");
		introX = introNode.getFloatAttribute("introX");
		introY = introNode.getFloatAttribute("introY");
		introWidth = introNode.getFloatAttribute("introWidth");
		introHeight = introNode.getFloatAttribute("introHeight");

		XMLElement outroNode = xml.getChild("outro");

		outroSrc = outroNode.getStringAttribute("src");
		outroX = outroNode.getFloatAttribute("outroX");
		outroY = outroNode.getFloatAttribute("outroY");
		outroWidth = outroNode.getFloatAttribute("outroWidth");
		outroHeight = outroNode.getFloatAttribute("outroHeight");

		XMLElement windowsNode = xml.getChild("windows");

		rows = windowsNode.getIntAttribute("rows");
		columns = windowsNode.getIntAttribute("columns");

		windows = new BWindow[windowsNode.getChildCount()];

		for (int i = 0; i < windowsNode.getChildCount(); i++) {
			XMLElement windowNode = windowsNode.getChild(i);

			int id = windowNode.getIntAttribute("id");

			float x = windowNode.getFloatAttribute("x");
			float y = windowNode.getFloatAttribute("y");
			float w = windowNode.getFloatAttribute("width");
			float h = windowNode.getFloatAttribute("height");

			int r = windowNode.getIntAttribute("rows");
			int c = windowNode.getIntAttribute("columns");

			int hasCamera = windowNode.getIntAttribute("hasCamera");

			windows[i] = new BWindow(parent, id, x, y, w, h, r, c, hasCamera);

			if (hasCamera == 1)
				cameraWindowId = id;
		}

		int windowsWidthSum = 0;
		int windowsHeightSum = 0;

		for (int i = 0; i < windows.length; i++) {
			if (windows[i].getX() == 0)
				windowsHeightSum += windows[i].getHeight();

			if (windows[i].getY() == 0)
				windowsWidthSum += windows[i].getWidth();
		}

		width = windowsWidthSum;
		height = windowsHeightSum;
	}

	public void drawBounds() {
		for (int i = 0; i < windows.length; i++)
			windows[i].drawBounds();
	}

	public void drawViewableBounds() {
		for (int i = 0; i < windows.length; i++)
			windows[i].drawViewableBounds();
	}

	public void drawIds() {
		for (int i = 0; i < windows.length; i++)
			windows[i].drawId();
	}

	public void scale(float _w, float _h) {
		float ratioX = _w / width;
		float ratioY = _h / height;

		float scale = ratioX < ratioY ? ratioX : ratioY;

		for (int i = 0; i < windows.length; i++)
			windows[i].setScale(scale);
	}

	public String getName() {
		return name;
	}

	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	public float getWidth() {
		return width;
	}

	public float getHeight() {
		return height;
	}

	public float getScale() {
		return scale;
	}

	public float getScaledX() {
		return x * scale;
	}

	public float getScaledY() {
		return y * scale;
	}

	public float getScaledWidth() {
		return width * scale;
	}

	public float getScaledHeight() {
		return height * scale;
	}

	public float getWindowCount() {
		return windows.length;
	}

	public float getRowCount() {
		return rows;
	}

	public float getColumnCount() {
		return columns;
	}

	public BWindow[] getWindows() {
		return windows;
	}

	public BWindow getWindow(int _windowId) {
		return windows[_windowId];
	}

	public float[] getWindowSize(int _windowId) {
		return new float[] { windows[_windowId].getWidth(),
				windows[_windowId].getHeight() };
	}

	public float[] getWindowPosition(int _windowId) {
		return new float[] { windows[_windowId].getX(),
				windows[_windowId].getY() };
	}

	public int getCameraWindowId() {
		return cameraWindowId;
	}

	public float[] getCameraWindowSize() {
		return new float[] { windows[cameraWindowId].getWidth(),
				windows[cameraWindowId].getHeight() };
	}

	public float[] getCameraWindowPosition() {
		return new float[] { windows[cameraWindowId].getX(),
				windows[cameraWindowId].getY() };
	}
}
