package ijeoma.fabrica.benetton.window;

import processing.core.PApplet;
import processing.core.PFont;
import processing.core.PImage;

public class CopyOfBWindow {
	PFont f;
	PApplet parent;
	private int id;
	private float x, y, width, height;
	private int rows, columns;
	private float viewableX, viewableY;
	private float viewableWidth, viewableHeight;
	private float scale;
	private boolean hasCamera;

	public static String SCALE_NORMAL = "scaleNormal";
	public static String SCALE_WIDTH = "scaleWidth";
	public static String SCALE_HEIGHT = "scaleHeight";

	String scaleMode = SCALE_NORMAL;

	public static String V_ALIGN_TOP = "alignTop";
	public static String V_ALIGN_CENTER = "alignCenter";
	public static String V_ALIGN_BOTTOM = "alignBottom";

	public static String H_ALIGN_LEFT = "alignLeft";
	public static String H_ALIGN_CENTER = "alignCenter";
	public static String H_ALIGN_RIGHT = "alignRight";

	String horizontalAlignMode = H_ALIGN_CENTER;
	String verticalAlignMode = V_ALIGN_CENTER;
	
	float ratioToWindowWidth, ratioToWindowHeight;

	float scaleToWindow = 1;
	float scaledToWindowWidth, scaledToWindowHeight;

	float alignedX = 0, alignedY = 0;

	CopyOfBWindow(PApplet _parent, int _id, float _x, float _y, float _width,
			float _height, int _rows, int _columns, int _hasCamera) {
		parent = _parent;

		f = parent.createFont("Arial", 32);

		id = _id;

		x = _x;
		y = _y;

		width = _width;
		height = _height;

		rows = _rows;
		columns = _columns;

		float rowsMax = (((rows / 5f) > (int) (rows / 5f)) ? (int) (rows / 5f) + 1
				: (int) (rows / 5f)) * 5;
		float columnsMax = (((columns / 5f) > (int) (columns / 5f)) ? (int) (columns / 5f) + 1
				: (int) (columns / 5f)) * 5;

		viewableX = x;// + (width * (columnsMax - columns) / columnsMax));
		viewableY = y + (height * ((rowsMax - rows) / rowsMax));

		viewableWidth = width * (columns / columnsMax);
		viewableHeight = height * (rows / rowsMax);

		scale = 1;

		ratioToWindowWidth = getViewableWidth() / 1;
		ratioToWindowHeight = getViewableHeight() / 1;

		scaleToWindow = 1;
		
		scaledToWindowWidth = 1 * scaleToWindow;
		scaledToWindowHeight = 1 * scaleToWindow;

		alignedX = 0;
		alignedY = 0;
		
		hasCamera = (_hasCamera == 1) ? true : false;
	}

	void drawBounds() {
		parent.strokeCap(PApplet.ROUND);
		parent.strokeWeight(4f);

		parent.stroke(255, 0, 0);
		parent.fill(0);
		parent.rect(getScaledX(), getScaledY(), getScaledWidth(),
				getScaledHeight());

		parent.strokeWeight(1);
		parent.noStroke();
		parent.noFill();
	}

	void drawViewableBounds() {
		parent.strokeCap(PApplet.ROUND);
		parent.strokeWeight(2f);

		parent.stroke(0, 255, 0);
		parent.fill(255);
		// parent.noFill();
		parent.rect(getScaledViewableX(), getScaledViewableY(),
				getScaledViewableWidth(), getScaledViewableHeight());

		parent.strokeWeight(1);
		parent.noStroke();
		parent.noFill();
	}

	void drawId() {
		parent.noStroke();
		parent.fill(0, 255, 0);
		parent.textFont(f, 32f);
		parent.text(id, getScaledViewableX() + getScaledViewableWidth() / 2
				- parent.textWidth(id + "") / 2, getScaledViewableY()
				+ getScaledViewableHeight() / 2);

		parent.strokeWeight(1);
		parent.noStroke();
		parent.noFill();
	}

	public boolean hasCamera() {
		return hasCamera;
	}

	public float[] mapPosition(float _x, float _y) {
		float mappedX = getScaledViewableX()
				+ PApplet.map(_x, 0, getViewableWidth(), 0,
						getScaledViewableWidth());

		float mappedY = getScaledViewableY()
				+ PApplet.map(_y, 0, getViewableHeight(), 0,
						getScaledViewableHeight());

		return new float[] { mappedX, mappedY };
	}

	public float[] mapSize(float _width, float _height) {
		float ratioToWindowWidth = getViewableWidth() / _width;
		float ratioToWindowHeight = getViewableHeight() / _height;

		float scaleToWindow = ratioToWindowWidth < ratioToWindowHeight ? ratioToWindowWidth
				: ratioToWindowHeight;

		float scaledToWindowWidth = _width * scaleToWindow;
		float scaledToWindowHeight = _height * scaleToWindow;

		return new float[] { scaledToWindowWidth * scale,
				scaledToWindowHeight * scale };
	}

	public float mapScale(float _width, float _height) {
		float ratioToWindowWidth = getViewableWidth() / _width;
		float ratioToWindowHeight = getViewableHeight() / _height;

		float scaleToWindow = 1;

		if (scaleMode == SCALE_NORMAL)
			scaleToWindow = ratioToWindowWidth < ratioToWindowHeight ? ratioToWindowWidth
					: ratioToWindowHeight;
		if (scaleMode == SCALE_WIDTH)
			scaleToWindow = ratioToWindowWidth;
		if (scaleMode == SCALE_HEIGHT)
			scaleToWindow = ratioToWindowHeight;

		return scaleToWindow * scale;
	}

	public int[] mapImageSize(PImage _image) {
		float[] scaledSize = mapSize(_image.width, _image.height);

		return new int[] { (int) scaledSize[0], (int) scaledSize[1] };
	}

	public PImage mapImage(PImage _image) {
		int[] scaledSize = mapImageSize(_image);

		_image.resize(scaledSize[0], scaledSize[1]);

		return _image;
	}

	public void applyMatrix() {
		if (scaleMode == SCALE_NORMAL)
			scaleToWindow = ratioToWindowWidth < ratioToWindowHeight ? ratioToWindowWidth
					: ratioToWindowHeight;
		else if (scaleMode == SCALE_WIDTH)
			scaleToWindow = ratioToWindowWidth;
		else if (scaleMode == SCALE_HEIGHT)
			scaleToWindow = ratioToWindowHeight;
		
		if (horizontalAlignMode == H_ALIGN_LEFT)
			alignedX = 0;
		else if (horizontalAlignMode == H_ALIGN_CENTER)
			alignedX = (getViewableWidth() / 2f - scaledToWindowWidth / 2f);
		else if (horizontalAlignMode == H_ALIGN_RIGHT)
			alignedX = (getViewableWidth() - scaledToWindowWidth);
		
		if (verticalAlignMode == V_ALIGN_TOP)
			alignedY = 0;
		else if (verticalAlignMode == V_ALIGN_CENTER)
			alignedY = (getViewableHeight() / 2f - scaledToWindowHeight / 2f);
		else if (verticalAlignMode == V_ALIGN_BOTTOM)
			alignedY = (getViewableHeight() - scaledToWindowHeight);
		
		float[] mappedAlignedPosition = mapPosition(alignedX, alignedY);
	}

	public void translate(float _x, float _y) {
		parent.rectMode(PApplet.CORNER);
		parent.imageMode(PApplet.CORNER);
		// parent.translate(mappedAlignedPosition[0], mappedAlignedPosition[1]);
	}

	public void scale(float _width, float _height) {
		parent.scale(mapScale(_width, _height));
	}

	public void transform(float _x, float _y, float _width, float _height) {
		float ratioToWindowWidth = getViewableWidth() / _width;
		float ratioToWindowHeight = getViewableHeight() / _height;

		float scaleToWindow = 1;

		if (scaleMode == SCALE_NORMAL)
			scaleToWindow = ratioToWindowWidth < ratioToWindowHeight ? ratioToWindowWidth
					: ratioToWindowHeight;
		if (scaleMode == SCALE_WIDTH)
			scaleToWindow = ratioToWindowWidth;
		if (scaleMode == SCALE_HEIGHT)
			scaleToWindow = ratioToWindowHeight;

		float scaledToWindowWidth = _width * scaleToWindow;
		float scaledToWindowHeight = _height * scaleToWindow;

		float alignedX = 0, alignedY = 0;

		if (horizontalAlignMode == H_ALIGN_LEFT)
			alignedX = 0;
		else if (horizontalAlignMode == H_ALIGN_CENTER)
			alignedX = (getViewableWidth() / 2f - scaledToWindowWidth / 2f);
		else if (horizontalAlignMode == H_ALIGN_RIGHT)
			alignedX = (getViewableWidth() - scaledToWindowWidth);

		if (verticalAlignMode == V_ALIGN_TOP)
			alignedY = 0;
		else if (verticalAlignMode == V_ALIGN_CENTER)
			alignedY = (getViewableHeight() / 2f - scaledToWindowHeight / 2f);
		else if (verticalAlignMode == V_ALIGN_BOTTOM)
			alignedY = (getViewableHeight() - scaledToWindowHeight);

		float[] mappedAlignedPosition = mapPosition(alignedX, alignedY);
		float mappedScale = mapScale(_width * scaleToWindow, _height
				* scaleToWindow);

		parent.rectMode(PApplet.CORNER);
		parent.imageMode(PApplet.CORNER);
		parent.translate(mappedAlignedPosition[0], mappedAlignedPosition[1]);
		parent.scale(mappedScale);
	}

	public void beginTranslate(float _x, float _y) {
		float[] mappedPosition = mapPosition(_x, _y);

		parent.pushMatrix();
		parent.translate(mappedPosition[0], mappedPosition[1]);
	}

	public void endTranslate() {
		parent.popMatrix();
	}

	public void beginScale(float _width, float _height) {
		parent.pushMatrix();
		parent.scale(mapScale(_width, _height));
	}

	public void endScale() {
		parent.popMatrix();
	}

	
	public int getId() {
		return id;
	}

	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	public float getWidth() {
		return width;
	}

	public float getHeight() {
		return height;
	}

	public float getViewableX() {
		return viewableX;
	}

	public float getViewableY() {
		return viewableY;
	}

	public float getViewableWidth() {
		return viewableWidth;
	}

	public float getViewableHeight() {
		return viewableHeight;
	}

	void setScale(float _value1) {
		scale = _value1;
	}

	public float getScale() {
		return scale;
	}

	public float getScaledX() {
		return x * scale;
	}

	public float getScaledY() {
		return y * scale;
	}

	public float getScaledWidth() {
		return width * scale;
	}

	public float getScaledHeight() {
		return height * scale;
	}

	public float getScaledViewableX() {
		return viewableX * scale;
	}

	public float getScaledViewableY() {
		return viewableY * scale;
	}

	public float getScaledViewableWidth() {
		return viewableWidth * scale;
	}

	public float getScaledViewableHeight() {
		return viewableHeight * scale;
	}

	public void setScaleMode(String _scaleMode) {
		scaleMode = _scaleMode;
	}

	public void setHorizantalAlignMode(String _alignMode) {
		horizontalAlignMode = _alignMode;
	}

	public void setVerticalAlignMode(String _alignMode) {
		verticalAlignMode = _alignMode;
	}
}
