package ijeoma.fabrica.benetton.window.test;

import ijeoma.fabrica.benetton.window.BWindow;
import ijeoma.fabrica.benetton.window.BWindowManager;

import processing.core.*;
import hypermedia.video.*;

public class Narcissus_Test extends PApplet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	BWindowManager bwm;

	PFont f;

	int MAX_BLOBS = 10, MAX_ANCHORS = 20;

	OpenCV opencv;

	int cvWidth = 160, cvHeight = 120;
	PVector cvScale;

	PGraphics blobsImage;

	public void setup() {
		size(1024, 768, P3D);

		f = createFont("Arial", 32);

		bwm = new BWindowManager(this, 0, 0);
		bwm.load("munich.xml");
		bwm.scale(width, height);

		BWindow bw1 = bwm.getWindow(0);
		bw1.setScaleMode(BWindow.SCALE_NORMAL);
		bw1.setHorizantalAlignMode(BWindow.H_ALIGN_CENTER);
		bw1.setVerticalAlignMode(BWindow.V_ALIGN_CENTER);

		cvScale = new PVector(width / cvWidth, height / cvHeight);

		blobsImage = createGraphics(width, height, P3D);

		setupCV();
	}

	public void draw() {
		background(255);

		updateCV();
		updateBlobs();

		pushMatrix();
		(bwm.getWindow(0)).setupTransform(width, height);
		drawImageAndBlobs();
		popMatrix();

		pushMatrix();
		(bwm.getWindow(1)).setupTransform(width, height);
		drawImageAndBlobs();
		popMatrix();

		pushMatrix();
		(bwm.getWindow(2)).setupTransform(width, height);
		drawImageAndBlobs();
		popMatrix();

		pushMatrix();
		(bwm.getWindow(3)).setupTransform(width, height);
		drawImageAndBlobs();
		popMatrix();

		bwm.drawBounds();
		bwm.drawViewableBounds();
		bwm.drawIds();
	}

	void setupCV() {
		opencv = new OpenCV(this);
		opencv.allocate(cvWidth, cvHeight);
		opencv.capture(cvWidth, cvHeight);
		opencv.cascade(OpenCV.CASCADE_FRONTALFACE_ALT);
	}

	void updateCV() {
		opencv.read();
		opencv.absDiff();
		opencv.threshold(80);
		opencv.remember();
	}

	void updateBlobs() {

		Blob[] blobs = opencv.blobs(10, cvWidth * cvHeight / 2, MAX_BLOBS,
				true, MAX_ANCHORS);

		blobsImage.beginDraw();
		blobsImage.background(255, 0);
		blobsImage.strokeWeight(10);
		blobsImage.stroke(255, 25);
		blobsImage.noFill();
		blobsImage.beginShape();
		for (int i = 0; i < blobs.length; i++)
			for (int j = 0; j < blobs[i].points.length - 1; j++)
				blobsImage.vertex(blobs[i].points[j].x * cvScale.x,
						blobs[i].points[j].y * cvScale.y);
		blobsImage.endShape();

		blobsImage.noStroke();
		blobsImage.fill(255, 25);
		for (int i = 0; i < blobs.length; i++)
			for (int j = 0; j < blobs[i].points.length - 1; j++)
				blobsImage.ellipse(blobs[i].points[j].x * cvScale.x,
						blobs[i].points[j].y * cvScale.y, 50, 50);
		blobsImage.endDraw();
	}

	void drawImageAndBlobs() {
		// tint(255, 150);
		image(opencv.image(OpenCV.SOURCE), 0, 0, width, height);
		image(blobsImage, 0, 0);
	}
}
